#!/usr/bin/env python

# -*- coding: utf-8 -*-

"""
This script takes as input the fedmsg messages published under the topic
``ci.pipeline.allpackages-pr.complete`` and flag the corresponding PR in
dist-git with the result of the tests.

Authors:    Pierre-Yves Chibon <pingou@pingoured.fr>

"""
from __future__ import unicode_literals

import hashlib
import json
import os
import sys

import requests
from requests.packages.urllib3.util import retry

def main(msg, pipeline_state='complete', seed='empty'):
    """ Check if the build was successful and if so, flag the commit in
    pagure.

    :param str msg: the fedmsg message body
    :param str pipeline_state: state of the testing, one of 'complete'
                               or 'running'
    """

    timeout = (30, 30)
    retries = 3
    requests_session = requests.Session()
    retry_conf = retry.Retry(
        total=retries, connect=retries, read=retries, backoff_factor=1)
    retry_conf.BACKOFF_MAX = 5
    requests_session.mount(
        'http://', requests.adapters.HTTPAdapter(max_retries=retry_conf))
    requests_session.mount(
        'https://', requests.adapters.HTTPAdapter(max_retries=retry_conf))

    if pipeline_state not in ['complete', 'running', 'error']:
        print("Pipeline state is not 'complete' or 'running' or 'error'.")
        return

    if 'version' not in msg or not msg['version'].startswith('0.2.'):
        print('Unsupported msg version, ignoring')
        return

    # test complete messages
    if pipeline_state == 'complete':
        done_states = {
            'passed': {'api': 'success', 'human': 'passed'},
            'error': {'api': 'error', 'human': 'errored'},
            'failed': {'api': 'failure', 'human': 'failed'},
            'needs_inspection': {'api': 'failure', 'human': 'failed'},
        }
        state = msg['test']['result']
        if state not in done_states:
            print('Build is not in one of the expected status, ignoring')
            return

        status = done_states[state]['api']
        human_status = done_states[state]['human']
    # test running messages
    elif pipeline_state == 'running':
        status = 'pending'
        human_status = 'running'

    # test error messages
    elif pipeline_state == 'error':
        status = 'error'
        human_status = 'errored'

    if 'id' not in msg['artifact']:
        print(
            'Invalid message: %s, could not extract the PR id from it' %
            msg['artifact'])
        return
    pr_id = str(msg['artifact']['id'])

    commit_hash_text = ' for %s' % msg['artifact']['commit_hash'][:8]

    data = {
        'username': 'Fedora CI',
        'status': status,
        'comment': 'Package tests%s: %s' % (commit_hash_text, human_status),
        'url': msg['run']['url'],
        'uid': hashlib.md5(pr_id + seed).hexdigest()
    }

    pagure_url = 'https://src.fedoraproject.org'
    if "stage" in msg["pipeline"]["name"]:
        pagure_url = 'https://src.stg.fedoraproject.org'

    env_var = 'API_TOKEN'

    namespace = msg['artifact']['repository'].split('/')[-2]
    repo = msg['artifact']['repository'].split('/')[-1]

    target_url = '/'.join([
        'api',
        '0',
        namespace,
        repo,
        'pull-request',
        pr_id,
        'flag'
    ])
    flag_url = pagure_url + '/' + target_url
    print('Flagging commit at: %s' % flag_url)

    headers = {
        "Authorization": "token " + os.environ.get(env_var),
        'User-Agent': 'loopabull@fedora-infra',
    }

    print('payload: %s' % data)

    req = requests_session.request(
        method='POST',
        url=flag_url,
        headers=headers,
        data=data,
    )
    print('Request to %s returned: %s' % (pagure_url, req.status_code))
    if not req.ok:
        print(req.text)
        return 1
    else:
        print('All clear')
        print('User-URL: %s' % msg['artifact']['repository'] + '/'.join([
            'pull-request',
            pr_id,
        ]))


if __name__ == '__main__':
    msg = sys.argv[1]
    msg = json.loads(msg)
    main(msg, sys.argv[2], sys.argv[3])
    sys.exit(0)
